#!/usr/bin/perl -T
# $Id: chrono_conf.cgi,v 1.14 2013/08/27 12:19:13 sofc Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Karin Soeren Koch           ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2005              ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use ElchemeaHeader;
use ElchemeaUser;
use Elchemea;
use ElchemeaCGI qw(:DEFAULT :html :cgi);
use CGI qw(:standard :netscape);


if (param('ajax'))
{
    print header();
    unless (login_ok())
    {
	print "";
	exit;
    }
    my $a = param('action');
    exit;
}


my $user;
my $bc_admin = undef;
if (!defined(param))
{
    print_header("Please log in to the Elchemea");
    print '<center>';
    &ElchemeaCGI::login;
    print '</center>';
    &print_end;
}
else
{
    if (login_ok())
    { 
	$user = ElchemeaUser->new(param('user'));
	&ElchemeaCGI::print_header("Elchemea chrono amperometry/potentiometry setup",-onload=>'load_all()');   
	top_nav_bar_start();
	param(-name=>'action',-value=>'') unless (param('action'));
	$_ = param('action');
      CASE:
	{
	    ### Insert case statements ###
	    ### Default response ####
	    &main_page;
	}
	print_end();
    }
    else 
    {
	not_auth();
    }
}
    
exit;

sub main_page
{
    top_nav_bar_button('config.cgi','','','Manually edit configuration','s6');
    top_no_button();
    top_no_button();
    tab_newrow();
    js_back();
    top_js_return('#006600','white');
    top_no_button();
    ElchemeaCGI::create_menu_field;
    print start_form(-action=>'run_chrono.cgi',-name=>'run_chrono',-class=>'form',-onsubmit=>'return check_run()');
    print_hidden();
    print submit(-name=>'action',-value=>'Run chrono',-class=>'none',-id=>'run_chrono_btn');
    print end_form;
    top_nav_bar_button('view_session.cgi','','','Session view','s6');
    top_nav_bar_button('main.cgi','','','To menu','s4');
    end_top_bar();
    my $pot = Elchemea::new_PSTAT(Elchemea::get_config_value('equipment','potentiostat'),Elchemea::get_config_value('equipment','potentiostat_address'));
    my $fra = Elchemea::new_FRA(Elchemea::get_config_value('equipment','FRA'),Elchemea::get_config_value('equipment','FRA_address'));
    print '<center>';
    print h1("Elchemea chrono amperometry/potentiometry setup");
    print '<table cellspacing=5 cellpadding=5 class=tabel"><tr>';
    print '<td width="5%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="5%"></td>';
    print '</tr>';
    my @cf = ('sleeptime','totaltime','keep_potential');
    my @list = $pot->setup_tags_chrono;
    my %setup = ();
    foreach my $tag (@list,@cf,'totaltime_unit')
    {
	$tag .= '_program' if ($tag =~ /offset/);
	$setup{$tag} = $user->get_config_value('chrono',$tag);
	$setup{$tag} = $pot->get_default($tag) unless (defined($setup{$tag}));
    }
    $setup{'totaltime_unit'} = 's' unless ($setup{'totaltime_unit'});
    my $count = 0;
    my @tags = ();
    my @yn = ('','Yes','No');
    my @offset_progs = Elchemea::get_dir_list('/usr/local/bin/elchemea/user_exec/','\w');
    unshift @offset_progs,'None';
    foreach my $key (@list,@cf)
    {
	my %labels = ();
	print '<tr><td></td>' unless ($count);
        print "<th onclick='get_desc(\"POT\",\"$key\")'>";
	my $tag = $key;
	$tag = 'filter' if ($tag eq 'bandwidth');
	$tag =~ s/_/ /g;
	$tag =~ s/(.*\w)(time)/$1 $2/;
	print ucfirst($tag);
	print '</th><td>';
	my @l = $pot->get_tag_values($key);
	@l = $pot->get_modes() if ($key eq 'mode');
	%labels = ('Volt' => 'Constant voltage', 'Amp' => 'Constant current') if ($key eq 'mode');
	@l = @offset_progs if ($key =~ /offset_prog/);
	@l = @yn if ($key eq 'keep_potential');
	if (@l>1 || $key =~ /offset_prog/)
	{
	    print popup_menu(-name=>$key,-values=>[(@l)],-class=>'textfield',-default=>$setup{$key},-force=>1,-onchange=>'change_val(this)',-id=>$key,-labels=>\%labels);
	}
	else
	{
	    print textfield(-name=>$key,-value=>$setup{$key},-class=>'textfield',-force=>1,-onchange=>'change_val(this)',-id=>$key);
	}
	print '</td><th id="tag_'.$key.'">';
        $setup{'mode'} =~ /^(\w)/;
        my $mk = $1;
	$_ = $tag;
      CASE:
	{
	    /polarisation$/ and do {print $mk;push @tags,"tag_$key";};
	    /resis/i and do {print 'Ohm'};
	    /cur/i and do {print 'A'};
	    /range/i and do {print 'V'};
	    /total time/i and do {print popup_menu(-name=>'totaltime_unit',-values=>['s','m','h'],-default=>$setup{'totaltime_unit'},-class=>'testfield',-force=>1,-onchange=>'change_val(this)',-id=>'totaltime_unit'); last CASE;};
	    /delay|time/ and do {print 's'};
	}
	print '</th>';
	if ($count++ > 1)
	{
	    $count = 0;
	    print '</tr>',"\n";
	}
    }
    my $valuearr = "var vals = [];\n";

    for (my $i = 0; $i < @tags; $i++ )
    {
	$valuearr .= 'vals['.$i.'] = "'.$tags[$i].'";'."\n";
    }


    print '</table>';
    print '</center>';
    print <<END;

<script language="javascript">

$valuearr

var notready = 0;
var running = 1;

function check_run()
{
    if (notready > 0)
    {
	return false;
    }
    if (running > 0)
    {
	return false;
    }
    return true;
}

function set_run()
{
    var obj = document.getElementById('run_chrono_btn');
    if (notready == 0 && running == 0)
    {
	obj.setAttribute("class","s3");	
        obj.setAttribute("className","s3");
    }
    else
    {
	obj.setAttribute("class","none");	
        obj.setAttribute("className","none");
    }
}

function set_status(s)
{
    var res = s.split("\\n");
    if (res[0] == "Free")
    {
	running = 0;
    }
    else
    {
	running = 1;
    }
    set_run();
}

function load_status()
{
    run_ajax('runstatus.cgi','&action=status',set_status);
    setTimeout("load_status()",1000);
}

function load_all()
{
    load_status();
}

function set_val(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK" || arr[0] == "WARN")
    {
        if (arr[0] == "WARN")
        {
            alert(arr[4]);
        }
	var obj = document.getElementById(arr[1]);
	obj.disabled = false;
	notready = notready-1;
	if (arr[1] == "mode")
	{
	    for (var i=0; i<vals.length;i++)
	    {
		var t = arr[2].split('');
		document.getElementById(vals[i]).innerHTML = t[0];
	    }
	}
        obj.value = arr[2];
    }
    else
    {
	alert(arr[0]);
    }
    set_run();
}

function get_desc(dev,key)
{
    run_ajax('ajax_desc.cgi','&key='+key+'&device='+dev,alert);
}


function change_val(element)
{
    var n = element.name;
    var val = element.value;
    element.disabled = true;
    notready = notready+1;
    set_run();
    run_ajax('change_config.cgi','&action=set_value&section=chrono&key='+n+'&value='+val,set_val);
}

</script>

END

}
